# Load required packages
library(ggplot2)
library(grid)

bmData <- data.frame()

# Read the data
for (node in c("01","04","09","16")) {
  filename=paste("time-",node,"-node-smic.dat",sep="")
  timeData <- read.table(filename,fill=TRUE,col.names=c("nt","time","nt2","time2"))
  # Recast the data frame to two columns ("nt" and "time") and add a third variable "device"
  phi <- subset(timeData, select=c(nt,time))
  phi$device <- "mic" # Add a device variable
  phi$speedup <- phi[1,"time"]/phi$time
  cpu <- subset(timeData, select=c(nt2,time2))
  colnames(cpu) <- c("nt","time")
  cpu$device <- "cpu" # Add a device variable
  cpu$speedup <- cpu[1,"time"]/cpu$time
  cpu <- cpu[complete.cases(cpu),] # Disregard the NA values
  recastData <- rbind.data.frame(phi,cpu) 
  recastData$node <- as.numeric(node) # Add "node" as a fourth variable
  bmData=rbind.data.frame(bmData,recastData) # 
}

# Plot 1
# Make a line plot with points, then write the plot to a pdf file

# width and height are in inches
pdf("smic_benchmark_1.pdf",height=12,width=12)

g <- ggplot(subset(bmData,device=="mic"), aes(nt,speedup,color=factor(node))) # Initial call of ggplot

g <- g + geom_line(size=1) + # Plot lines
  geom_point(size=5) +  # Plot points
  labs(x="Number of OpenMP Threads", y="Speedup", title="SuperMIC Benchmark") + # Title and axis labels
  theme(plot.title = element_text(colour="black", size=22, face="bold", vjust=2.5), # Format title
        legend.title = element_text(colour="black", size=16, face="bold"), # Format and position legends
        legend.position=c(0.2,0.8),
        legend.text=element_text(face="italic", size=16), 
        axis.title=element_text(size=16)) + # Format axis labels
  scale_color_discrete(name="Offload to Intel Xeon Phi", # Legend title and keys
        labels=c("1 node", "4 nodes", "9 nodes", "16 nodes")) +
  scale_x_continuous(limits=c(1, max(bmData$nt)), # Range and ticks for x axis
        breaks=c(1,seq(20,max(bmData$nt),20))) +
  scale_y_continuous(limits=c(1, max(bmData$speedup)), # Range and ticks for y axis
        breaks=c(1,seq(20,max(bmData$speedup),20))) +
  annotate("text", x=200, y=30, # Annotation
        label="MIC KMP AFFINITY=\n\"granularity=fine,balanced\"", size=8)

print(g)

dev.off()

# Plot 2
# Make a plot with inset, then write to a svg file

svg("smic_benchmark_2.svg", width=12, height=12)

g <- ggplot(subset(bmData, node==16), aes(nt,speedup,color=factor(device))) # Initial call of ggplot

print(g + geom_line(size=1) + # Plot lines
  geom_point(size=5) +  # Plot points
  labs(x="Number of OpenMP Threads", y="Speedup", title="SuperMIC Benchmark") + # Title and axis labels
  theme(plot.title = element_text(colour="black", size=22, face="bold", vjust=2.5), # Format title
        legend.title = element_text(colour="black", size=16, face="bold"), # Format and position legends
        legend.position=c(0.2,0.8),
        legend.text=element_text(face="italic", size=16), 
        axis.title=element_text(size=16)) + # Format axis labels
  scale_color_discrete(name="16 MPI Tasks on 16 Nodes", 
        labels=c("Intel Xeon Phi", "Intel Xeon")) +
  scale_x_continuous(limits=c(1, max(bmData$nt)), # Range and ticks for x axis
                     breaks=c(1,seq(20,max(bmData$nt),20))) +
  scale_y_continuous(limits=c(1, max(bmData[bmData$node==16,]$speedup)), # Range and ticks for y axis
                     breaks=c(1,seq(20,max(bmData$speedup),20))) )

vp1 <- viewport(width=0.5, height=0.3, x=0.65, y=0.3) # Define the size and the position for a inset as a viewport.

# Print the inset
print(g + geom_line(size=1) + # Plot lines
        geom_point(size=5) + # Plot points
        scale_x_continuous(limits=c(1, 22), # Range and ticks for x axis
                           breaks=c(1,seq(5,30,5))) +
        scale_y_continuous(limits=c(1, 20), # Range and ticks for y axis
                           breaks=c(1,seq(5,20,5))) +
        theme(legend.position="none", # Turn off the legend
              axis.title=element_blank(), # Turn off the axis labels
              plot.margin=unit(c(0.5,0.5,0,0.),"line")) # Define the plot margin
      , vp = vp1)

dev.off()

# Plot 3
# Make a plot with a 2x2 facet, then write to a png file.

# The unit of the dimensions of a device can be specified.
png("smic_benchmark_3.png",height=12,width=12,units="in",res=144)

g <- ggplot(bmData, aes(nt,speedup,color=factor(device))) # Initial call of ggplot

print (g + geom_line(size=1) + # Plot lines
  geom_point(size=2) +  # Plot points
  facet_wrap(~ node, nrow=2) + # Create a 2x2 grid of plots
  labs(x="Number of OpenMP Threads", y="Speedup", title="SuperMIC Benchmark") + # Title and axis labels
  theme(plot.title = element_text(colour="black", size=22, face="bold", vjust=2.5), # Format title
        legend.title = element_blank(), # Format and position legends
        #legend.position=c(0.2,0.8),
        legend.text=element_text(face="italic", size=16), 
        axis.title=element_text(size=16)) + # Format axis labels
  scale_color_discrete(#name="16 MPI Tasks on 16 Nodes", 
                       labels=c("Intel Xeon Phi", "Intel Xeon")) +
  scale_x_continuous(limits=c(1, max(bmData$nt)), # Range and ticks for x axis
                     breaks=c(1,seq(20,max(bmData$nt),20))) +
  scale_y_continuous(limits=c(1, max(bmData$speedup)), # Range and ticks for y axis
                     breaks=c(1,seq(20,max(bmData$speedup),20))) )

dev.off()